@@ -1,16 +1,19 @@
 """
-:Author: Pauli Virtanen <pav@iki.fi>
-:Date: 2008-04-26
-:Author: Pit Garbe <piiit@gmx.de>
-:Date: 2014-02-03
-:Author: TexText developers
-:Date: 2019-04-05
-:License: BSD
+This file is part of TexText, an extension for the vector
+illustration program Inkscape.
 
-This is the GUI part of TexText, handling several more or less sophisticated dialog windows
- depending on the installed tools.
+Copyright (c) 2006-2022 TexText developers.
 
-Its used uniformly from textext.py via the factory (AskerFactory) and only the "ask" method is called.
+TexText is released under the 3-Clause BSD license. See
+file LICENSE.txt or go to https://github.com/textext/textext
+for full license details.
+
+This is the GUI part of TexText, handling several more or less
+sophisticated dialog windows depending on the installed tools.
+
+It is used uniformly from base.py via the ask method of the
+AskText class depending on the available GUI framework
+(TkInter or GTK3).
 """
 
 DEBUG = False
@@ -54,9 +57,31 @@ try:
         os.environ['GI_TYPELIB_PATH'] = os.path.dirname(os.path.abspath(__file__))
 
     import gi
-
     gi.require_version("Gtk", "3.0")
-    from gi.repository import Gtk
+
+    # The import statement
+    # from gi.repository import Gtk
+    # writes a warning into stderr under Python 3.10 which always pops up after the
+    # extensions has been executed:
+    # "DynamicImporter.exec_module() not found; falling back to load_module()"
+    # We redirect stderr here into a string, check if
+    # this warning has been writen and silently discard it. If something else has
+    # been written to stderr we pass it to stderr.
+    # Related issues:
+    # https://gitlab.com/inkscape/extensions/-/issues/463
+    # ToDo: Remove the stuff around the import statement when this has been fixed in
+    #       updated Python 3.10 releases or is properly handled by Inkscape
+    # ======
+    from contextlib import redirect_stderr
+    import io
+    with redirect_stderr(io.StringIO()) as h:
+        from gi.repository import Gtk
+
+    # Sort out messages matching the ImportWarning, keep all others and send them to stderr
+    for msg in (val for val in h.getvalue().splitlines(keepends=True)
+                    if val and val.find("ImportWarning: DynamicImporter") == -1):
+        sys.stderr.write(msg)
+    # ======
 
     from gi.repository import Gdk, GdkPixbuf
 
@@ -87,15 +112,15 @@ except (ImportError, TypeError, ValueError) as _:
                            "installation instructions on https://textext.github.io/textext/ !")
 
 
-def set_monospace_font(text_view):
+def set_monospace_font(text_view, font_size):
     """
     Set the font to monospace in the text view
     :param text_view: A GTK TextView
+    :param font_size: The font size in the TextView in pt
     """
     try:
-        import pango
-
-        font_desc = pango.FontDescription('monospace 11')
+        from gi.repository import Pango
+        font_desc = Pango.FontDescription('monospace %d' % (font_size))
         if font_desc:
             text_view.modify_font(font_desc)
     except ImportError:
@@ -113,15 +138,17 @@ class AskText(object):
     DEFAULT_AUTOINDENT = True
     DEFAULT_INSERTSPACES = True
     DEFAULT_TABWIDTH = 4
+    DEFAULT_FONTSIZE = 11
     DEFAULT_NEW_NODE_CONTENT = "Empty"
     DEFAULT_CLOSE_SHORTCUT = "Escape"
     DEFAULT_CONFIRM_CLOSE = True
     DEFAULT_PREVIEW_WHITE_BACKGROUND = False
+    FONT_SIZE = [11, 12, 14, 16]
     NEW_NODE_CONTENT = ["Empty", "InlineMath", "DisplayMath"]
     CLOSE_SHORTCUT = ["Escape", "CtrlQ", "None"]
 
     def __init__(self, version_str, text, preamble_file, global_scale_factor, current_scale_factor, current_alignment,
-                 current_texcmd, tex_commands, gui_config):
+                 current_texcmd, current_convert_strokes_to_path, tex_commands, gui_config):
         self.TEX_COMMANDS = tex_commands
         if len(text) > 0:
             self.text = text
@@ -142,6 +169,8 @@ class AskText(object):
         else:
             self.current_texcmd = self.TEX_COMMANDS[0]
 
+        self.current_convert_strokes_to_path = current_convert_strokes_to_path
+
         self.preamble_file = preamble_file
         self._preamble_widget = None
         self._scale = None
@@ -199,9 +228,9 @@ class AskTextTK(AskText):
     """TK GUI for editing TexText objects"""
 
     def __init__(self, version_str, text, preamble_file, global_scale_factor, current_scale_factor, current_alignment,
-                 current_texcmd, tex_commands, gui_config):
+                 current_texcmd, current_convert_strokes_to_path, tex_commands, gui_config):
         super(AskTextTK, self).__init__(version_str, text, preamble_file, global_scale_factor, current_scale_factor,
-                                        current_alignment, current_texcmd, tex_commands, gui_config)
+                                        current_alignment, current_texcmd, current_convert_strokes_to_path, tex_commands, gui_config)
         self._frame = None
         self._scale = None
 
@@ -256,19 +285,32 @@ class AskTextTK(AskText):
                                        command=self.select_preamble_file)
         self._askfilename_button.pack(ipadx=10, ipady=4, pady=5, padx=5, side="left")
 
-        box.pack(fill="x", pady=5, expand=True)
+        box.pack(fill="x", pady=0, expand=True)
+
+        # Frame holding the advanced settings and the tex command
+        box2 = Tk.Frame(self._frame, relief="flat")
+        box2.pack(fill="x", pady=5, expand=True)
+
+        # Frame box for advanced settings
+        self._convert_strokes_to_path = Tk.BooleanVar()
+        self._convert_strokes_to_path.set(self.current_convert_strokes_to_path)
+        box = Tk.Frame(box2, relief="groove", borderwidth=2)
+        label = Tk.Label(box, text="SVG-output:")
+        label.pack(pady=2, padx=5, anchor="w")
+        Tk.Checkbutton(box, text="No strokes", variable=self._convert_strokes_to_path, onvalue=True, offvalue=False).pack(side="left", expand=False, anchor="w")
+        box.pack(side=Tk.RIGHT, fill="x", pady=5, expand=True)
 
         # Frame box for tex command
         self._tex_command_tk_str = Tk.StringVar()
         self._tex_command_tk_str.set(self.current_texcmd)
-
-        box = Tk.Frame(self._frame, relief="groove", borderwidth=2)
+        box = Tk.Frame(box2, relief="groove", borderwidth=2)
         label = Tk.Label(box, text="TeX command:")
         label.pack(pady=2, padx=5, anchor="w")
         for tex_command in self.TEX_COMMANDS:
             Tk.Radiobutton(box, text=tex_command, variable=self._tex_command_tk_str,
                            value=tex_command).pack(side="left", expand=False, anchor="w")
-        box.pack(fill="x", pady=5, expand=True)
+        box.pack(side=Tk.RIGHT, fill="x", pady=5, expand=True)
+
 
         # Frame box for scale factor and reset buttons
         box = Tk.Frame(self._frame, relief="groove", borderwidth=2)
@@ -380,10 +422,11 @@ class AskTextTK(AskText):
             return
         self.text = self._text_box.get(1.0, Tk.END)
         self.preamble_file = self._preamble.get()
+        self.current_convert_strokes_to_path = self._convert_strokes_to_path.get()
 
         try:
             self.callback(self.text, self.preamble_file, self.global_scale_factor, self._alignment_tk_str.get(),
-                          self._tex_command_tk_str.get())
+                          self._tex_command_tk_str.get(), self.current_convert_strokes_to_path)
         except Exception as error:
             self.show_error_dialog("TexText Error",
                               "Error occurred while converting text from Latex to SVG:",
@@ -461,9 +504,10 @@ class AskTextGTKSource(AskText):
     """GTK + Source Highlighting for editing TexText objects"""
 
     def __init__(self, version_str, text, preamble_file, global_scale_factor, current_scale_factor, current_alignment,
-                 current_texcmd, tex_commands, gui_config):
+                 current_texcmd, current_convert_strokes_to_path, tex_commands, gui_config):
         super(AskTextGTKSource, self).__init__(version_str, text, preamble_file, global_scale_factor, current_scale_factor,
-                                               current_alignment, current_texcmd, tex_commands, gui_config)
+                                               current_alignment, current_texcmd, current_convert_strokes_to_path,
+                                               tex_commands, gui_config)
         self._preview = None  # type: Gtk.Image
         self._pixbuf = None  # type: GdkPixbuf
         self.preview_representation = "SCALE"  # type: str
@@ -482,6 +526,7 @@ class AskTextGTKSource(AskText):
                 ('FileMenu', None, '_File'),
                 ('ViewMenu', None, '_View'),
                 ('SettingsMenu', None, '_Settings'),
+                ('FontSize', None, 'Editor Font Si_ze'),
                 ('NewNodeContent', None, '_New Node Content'),
                 ('CloseShortcut', None, 'Close TexText _Shortcut'),
                 ('TabsWidth', None, '_Tabs Width'),
@@ -491,6 +536,7 @@ class AskTextGTKSource(AskText):
                 ('FileMenu', None, '_File'),
                 ('ViewMenu', None, '_View'),
                 ('SettingsMenu', None, '_Settings'),
+                ('FontSize', None, 'Editor Font Si_ze'),
                 ('NewNodeContent', None, '_New Node Content'),
                 ('CloseShortcut', None, '_Close TexText Shortcut'),
             ]
@@ -509,6 +555,15 @@ class AskTextGTKSource(AskText):
              'Wrap long lines in editor to avoid horizontal scrolling', self.word_wrap_toggled_cb)
         ]
 
+        self._font_size_actions = [
+            ('FontSize11', None, '1_1 pt', None, 'Set editor font size to 11pt', 0),
+            ('FontSize12', None, '1_2 pt', None, 'Set editor font size to 12pt', 1),
+            ('FontSize14', None, '1_4 pt', None, 'Set editor font size to 14pt', 2),
+            ('FontSize16', None, '1_6 pt', None, 'Set editor font size to 16pt', 3)
+        ]
+        font_size = "\n".join(
+            ['<menuitem action=\'%s\'/>' % action for (action, _, _, _, _, _) in self._font_size_actions])
+
         self._preview_white_background_action = [
             ('WhitePreviewBackground', None, 'White preview background', None,
              'Set preview background to white', self.on_preview_background_chagned)
@@ -556,6 +611,9 @@ class AskTextGTKSource(AskText):
               <menuitem action='Open'/>
             </menu>
             <menu action='ViewMenu'>
+              <menu action='FontSize'>
+                {font_size}
+              </menu>
               <menuitem action='WordWrap'/>
               {additions}
               <menuitem action='WhitePreviewBackground'/>
@@ -571,7 +629,7 @@ class AskTextGTKSource(AskText):
             </menu>
           </menubar>
         </ui>
-        """.format(additions=gtksourceview_ui_additions,
+        """.format(additions=gtksourceview_ui_additions, font_size=font_size,
                    new_node_content=new_node_content, close_shortcut=close_shortcut)
 
     @staticmethod
@@ -628,7 +686,8 @@ class AskTextGTKSource(AskText):
         """
 
         try:
-            text = open(path).read()
+            with open(path) as file_handle:
+                text = file_handle.read()
         except IOError:
             print("Couldn't load file: %s", path)
             return False
@@ -680,6 +739,10 @@ class AskTextGTKSource(AskText):
     def new_node_content_cb(self, action, previous_value, sourceview):
         self._gui_config["new_node_content"] = self.NEW_NODE_CONTENT[action.get_current_value()]
 
+    def font_size_cb(self, action, previous_value, sourceview):
+        self._gui_config["font_size"] = self.FONT_SIZE[action.get_current_value()]
+        set_monospace_font(sourceview, self._gui_config["font_size"])
+
     def close_shortcut_cb(self, action, previous_value, sourceview):
         self._gui_config["close_shortcut"] = self.CLOSE_SHORTCUT[action.get_current_value()]
         self._cancel_button.set_tooltip_text(
@@ -732,10 +795,13 @@ class AskTextGTKSource(AskText):
 
         self.global_scale_factor = self._scale_adj.get_value()
 
+        self.current_convert_strokes_to_path = self._conv_stroke2path.get_active()
+
         try:
             self.callback(self.text, self.preamble_file, self.global_scale_factor,
                           self.ALIGNMENT_LABELS[self._alignment_combobox.get_active()],
-                          self.TEX_COMMANDS[self._texcmd_cbox.get_active()].lower())
+                          self.TEX_COMMANDS[self._texcmd_cbox.get_active()].lower(),
+                          self.current_convert_strokes_to_path)
         except Exception as error:
             self.show_error_dialog("TexText Error",
                                    "Error occurred while converting text from Latex to SVG:",
@@ -1038,10 +1104,19 @@ class AskTextGTKSource(AskText):
 
         alignment_box.pack_start(self._alignment_combobox, True, True, 2)
 
-        # --- Scale and alignment together in one "line"
+        # Advanced settings
+        adv_settings_frame = Gtk.Frame()
+        adv_settings_frame.set_label("SVG output")
+        self._conv_stroke2path = Gtk.CheckButton(label="No strokes")
+        self._conv_stroke2path.set_tooltip_text("Ensures that strokes (lines, e.g. in \\sqrt, \\frac) can be easily \ncolored in Inkscape (Time consuming compilation!)")
+        self._conv_stroke2path.set_active(self.current_convert_strokes_to_path)
+        adv_settings_frame.add(self._conv_stroke2path)
+
+        # --- Scale, alignment and advanced settings together in one "line"
         scale_align_hbox = Gtk.HBox(homogeneous=False, spacing=0)
         scale_align_hbox.pack_start(scale_frame, False, False, 5)
         scale_align_hbox.pack_start(alignment_frame, True, True, 5)
+        scale_align_hbox.pack_start(adv_settings_frame, True, True, 5)
 
         # --- TeX code window ---
         # Scrolling Window with Source View inside
@@ -1070,7 +1145,7 @@ class AskTextGTKSource(AskText):
         self._source_view.set_size_request(-1, 150)
 
         scroll_window.add(self._source_view)
-        set_monospace_font(self._source_view)
+        set_monospace_font(self._source_view, self.DEFAULT_FONTSIZE)
 
         # Action group and UI manager
         ui_manager = Gtk.UIManager()
@@ -1083,6 +1158,7 @@ class AskTextGTKSource(AskText):
         action_group.add_actions(self.buffer_actions, text_buffer)
         action_group.add_radio_actions(self._new_node_content_actions, -1, self.new_node_content_cb, source_view)
         action_group.add_radio_actions(self._close_shortcut_actions, -1, self.close_shortcut_cb, source_view)
+        action_group.add_radio_actions(self._font_size_actions, -1, self.font_size_cb, source_view)
         action_group.add_toggle_actions(self._confirm_close_action, source_view)
         action_group.add_toggle_actions(self._word_wrap_action, source_view)
         action_group.add_toggle_actions(self._preview_white_background_action, source_view)
@@ -1136,6 +1212,7 @@ class AskTextGTKSource(AskText):
 
         vbox.show_all()
 
+        # ToDo: Currently this seems to do nothing?
         self._same_height_objects = [
             preamble_frame,
             texcmd_frame,
@@ -1148,6 +1225,9 @@ class AskTextGTKSource(AskText):
         groups = ui_manager.get_action_groups()
         # retrieve the view action group at position 0 in the list
         action_group = groups[0]
+        font_size_value = self._gui_config.get("font_size", self.DEFAULT_FONTSIZE)
+        action = action_group.get_action('FontSize{}'.format(font_size_value))
+        action.set_active(True)
         action = action_group.get_action('WordWrap')
         action.set_active(self._gui_config.get("word_wrap", self.DEFAULT_WORDWRAP))
         new_node_content_value = self._gui_config.get("new_node_content", self.DEFAULT_NEW_NODE_CONTENT)
@@ -1210,7 +1290,7 @@ class AskTextGTKSource(AskText):
             # create first window
             with SuppressStream():  # suppress GTK Warings printed directly to stderr in C++
                 window = self.create_window()
-            window.set_default_size(500, 500)
+            window.set_default_size(500, 525)
             # Until commit 802d295e46877fd58842b61dbea4276372a2505d we called own normalize_ui_row_heights here with
             # bad hide/show/hide hack, see issue #114
             window.show()
